class Series
  def initialize(symbol, params, agent)
    @symbol = symbol
    @params = params
    @agent = agent
  end

  INTERVAL_OPTIONS = ["m", "w", "d", "i1", "i3", "i5", "i15"]

  def run
    @agent.get(url)
    result
  end

  private

  def result
    [
      json["date"],
      json["volume"],
      json["open"],
      json["high"],
      json["low"],
      json["close"]
    ].transpose.map do |date, volume, open, close, high, low|
      {
        date: date,
        volume: volume,
        open: open,
        high: high,
        low: low,
        close: close
      }
    end
  end

  def json
    @json ||= JSON.parse(json_string)
  end

  def json_string
    html.match(/var data = ({.*?})/)[1]
  end

  def html
    @html ||= @agent.page.content
  end

  def interval
    if @params["interval"]
      throw "incorrect interval type" unless INTERVAL_OPTIONS.include?(@params["interval"])
      @params["interval"]
    else
      "d"
    end
  end

  def url
    "http://elite.finviz.com/quote.ashx?t=TNET&ty=c&ta=2&p=#{interval}&b=1"
  end
end
